@@ -1,8 +1,8 @@
 # coding: utf-8
 
-VERSION = (1, 1, 12)
-CODENAME = "opus"
-BUILD_DT = (2022, 1, 18)
+VERSION = (1, 3, 1)
+CODENAME = "god dag"
+BUILD_DT = (2022, 6, 16)
 
 S_VERSION = ".".join(map(str, VERSION))
 S_BUILD_DT = "{0:04d}-{1:02d}-{2:02d}".format(*BUILD_DT)
