@@ -135,7 +135,7 @@ function up2k_flagbus() {
 }
 
 
-function U2pvis(act, btns) {
+function U2pvis(act, btns, uc) {
     var r = this;
     r.act = act;
     r.ctr = { "ok": 0, "ng": 0, "bz": 0, "q": 0 };
@@ -425,7 +425,9 @@ function U2pvis(act, btns) {
                 html.push(r.genrow(a, true).replace(/><td>/, "><td>b "));
             }
         }
-        ebi('u2tab').tBodies[0].innerHTML = html.join('\n');
+        var el = ebi('u2tab');
+        el.tBodies[0].innerHTML = html.join('\n');
+        el.className = (uc.fsearch ? 'srch ' : 'up ') + r.act;
     };
 
     r.genrow = function (nfile, as_html) {
@@ -566,12 +568,12 @@ function Donut(uc, st) {
 
 function fsearch_explain(n) {
     if (n)
-        return toast.inf(60, 'your access to this folder is Read-Only\n\n' + (acct == '*' ? 'you are currently not logged in' : 'you are currently logged in as "' + acct + '"'));
+        return toast.inf(60, L.ue_ro + (acct == '*' ? L.ue_nl : L.ue_la).format(acct));
 
     if (bcfg_get('fsearch', false))
-        return toast.inf(60, 'you are currently in file-search mode\n\nswitch to upload-mode by clicking the green magnifying glass (next to the big yellow search button), and try uploading again\n\nsorry');
+        return toast.inf(60, L.ue_sr);
 
-    return toast.inf(60, 'try again, it should work now');
+    return toast.inf(60, L.ue_ta);
 }
 
 
@@ -585,19 +587,12 @@ function up2k_init(subtle) {
 
     function unmodal() {
         ebi('u2notbtn').style.display = 'none';
-        ebi('u2btn').style.display = 'block';
         ebi('u2conf').style.opacity = '1';
+        ebi('u2btn').style.display = '';
         ebi('u2notbtn').innerHTML = '';
     }
 
-    var suggest_up2k = 'this is the basic uploader; <a href="#" id="u2yea">up2k</a> is better';
-
-    var shame = 'your browser <a href="https://www.chromium.org/blink/webcrypto">disables sha512</a> unless you <a href="' + (window.location + '').replace(':', 's:') + '">use https</a>',
-        is_https = (window.location + '').indexOf('https:') === 0;
-
-    if (is_https)
-        // chrome<37 firefox<34 edge<12 opera<24 safari<7
-        shame = 'your browser is impressively ancient';
+    var suggest_up2k = L.u_su2k;
 
     function got_deps() {
         return subtle || window.asmCrypto || window.hashwasm;
@@ -606,15 +601,18 @@ function up2k_init(subtle) {
     var loading_deps = false;
     function init_deps() {
         if (!loading_deps && !got_deps()) {
-            var fn = 'sha512.' + sha_js + '.js';
-            showmodal('<h1>loading ' + fn + '</h1><h2>since ' + shame + '</h2><h4>thanks chrome</h4>');
+            var fn = 'sha512.' + sha_js + '.js',
+                m = L.u_https1 + ' <a href="' + (window.location + '').replace(':', 's:') + '">' + L.u_https2 + '</a> ' + L.u_https3;
+
+            showmodal('<h1>loading ' + fn + '</h1>');
             import_js('/.cpr/deps/' + fn, unmodal);
 
-            if (is_https)
-                ebi('u2foot').innerHTML = shame + ' so <em>this</em> uploader will do like 500 KiB/s at best';
-            else
-                ebi('u2foot').innerHTML = 'seems like ' + shame + ' so do that if you want more performance <span style="color:#' +
-                    (sha_js == 'ac' ? 'c84">(expecting 20' : '8a5">(but dont worry too much, expect 100') + ' MiB/s)</span>';
+            if (is_https) {
+                // chrome<37 firefox<34 edge<12 opera<24 safari<7
+                m = L.u_ancient;
+                setmsg('');
+            }
+            ebi('u2foot').innerHTML = '<big>' + m + '</big>';
         }
         loading_deps = true;
     }
@@ -624,11 +622,11 @@ function up2k_init(subtle) {
 
     function setmsg(msg, type) {
         if (msg !== undefined) {
-            ebi('u2err').setAttribute('class', type);
+            ebi('u2err').className = type;
             ebi('u2err').innerHTML = msg;
         }
         else {
-            ebi('u2err').setAttribute('class', '');
+            ebi('u2err').className = '';
             ebi('u2err').innerHTML = '';
         }
         if (msg == suggest_up2k) {
@@ -644,24 +642,8 @@ function up2k_init(subtle) {
         return false;
     }
 
-    ebi('u2nope').onclick = function (e) {
-        ev(e);
-        setmsg(suggest_up2k, 'msg');
-        goto('bup');
-    };
-
     setmsg(suggest_up2k, 'msg');
 
-    if (!String.prototype.format) {
-        String.prototype.format = function () {
-            var args = arguments;
-            return this.replace(/{(\d+)}/g, function (match, number) {
-                return typeof args[number] != 'undefined' ?
-                    args[number] : match;
-            });
-        };
-    }
-
     var parallel_uploads = icfg_get('nthread'),
         uc = {},
         fdom_ctr = 0,
@@ -671,8 +653,8 @@ function up2k_init(subtle) {
     bcfg_bind(uc, 'ask_up', 'ask_up', true, null, false);
     bcfg_bind(uc, 'flag_en', 'flag_en', false, apply_flag_cfg);
     bcfg_bind(uc, 'fsearch', 'fsearch', false, set_fsearch, false);
-    bcfg_bind(uc, 'turbo', 'u2turbo', false, draw_turbo, false);
-    bcfg_bind(uc, 'datechk', 'u2tdate', true, null, false);
+    bcfg_bind(uc, 'turbo', 'u2turbo', turbolvl > 1, draw_turbo, false);
+    bcfg_bind(uc, 'datechk', 'u2tdate', turbolvl < 3, null, false);
 
     var st = {
         "files": [],
@@ -711,7 +693,7 @@ function up2k_init(subtle) {
             });
     }
 
-    var pvis = new U2pvis("bz", '#u2cards'),
+    var pvis = new U2pvis("bz", '#u2cards', uc),
         donut = new Donut(uc, st);
 
     var bobslice = null;
@@ -719,7 +701,7 @@ function up2k_init(subtle) {
         bobslice = File.prototype.slice || File.prototype.mozSlice || File.prototype.webkitSlice;
 
     if (!bobslice || !window.FileReader || !window.FileList)
-        return un2k("this is the basic uploader; up2k needs at least<br />chrome 21 // firefox 13 // edge 12 // opera 12 // safari 5.1");
+        return un2k(L.u_ever);
 
     var flag = false;
     apply_flag_cfg();
@@ -741,14 +723,14 @@ function up2k_init(subtle) {
         var mup, up = QS('#up_zd');
         var msr, sr = QS('#srch_zd');
         if (!has(perms, 'write'))
-            mup = 'you do not have write-access to this folder';
+            mup = L.u_ewrite;
         if (!has(perms, 'read'))
-            msr = 'you do not have read-access to this folder';
+            msr = L.u_eread;
         if (!have_up2k_idx)
-            msr = 'file-search is not enabled in server config';
+            msr = L.u_enoi;
 
-        up.querySelector('span').textContent = mup || 'drop it here';
-        sr.querySelector('span').textContent = msr || 'drop it here';
+        up.querySelector('span').textContent = mup || L.udt_drop;
+        sr.querySelector('span').textContent = msr || L.udt_drop;
         clmod(up, 'err', mup);
         clmod(sr, 'err', msr);
         clmod(up, 'ok', !mup);
@@ -974,22 +956,22 @@ function up2k_init(subtle) {
     function gotallfiles(good_files, nil_files, bad_files) {
         var ntot = good_files.concat(nil_files, bad_files).length;
         if (bad_files.length) {
-            var msg = 'These {0} files (of {1} total) were skipped, possibly due to filesystem permissions:\n'.format(bad_files.length, ntot);
+            var msg = L.u_badf.format(bad_files.length, ntot);
             for (var a = 0, aa = Math.min(20, bad_files.length); a < aa; a++)
                 msg += '-- ' + bad_files[a][1] + '\n';
 
-            msg += '\nMaybe it works better if you select just one file';
+            msg += L.u_just1;
             return modal.alert(msg, function () {
                 gotallfiles(good_files, nil_files, []);
             });
         }
 
         if (nil_files.length) {
-            var msg = 'These {0} files (of {1} total) are blank/empty; upload them anyways?\n'.format(nil_files.length, ntot);
+            var msg = L.u_blankf.format(nil_files.length, ntot);
             for (var a = 0, aa = Math.min(20, nil_files.length); a < aa; a++)
                 msg += '-- ' + nil_files[a][1] + '\n';
 
-            msg += '\nMaybe it works better if you select just one file';
+            msg += L.u_just1;
             return modal.confirm(msg, function () {
                 gotallfiles(good_files.concat(nil_files), [], []);
             }, function () {
@@ -1003,12 +985,15 @@ function up2k_init(subtle) {
             return a < b ? -1 : a > b ? 1 : 0;
         });
 
-        var msg = ['{0} these {1} files?<ul>'.format(uc.fsearch ? 'search' : 'upload', good_files.length)];
+        var msg = [L.u_asku.format(good_files.length, esc(get_vpath())) + '<ul>'];
         for (var a = 0, aa = Math.min(20, good_files.length); a < aa; a++)
             msg.push('<li>' + esc(good_files[a][1]) + '</li>');
 
         if (uc.ask_up && !uc.fsearch)
-            return modal.confirm(msg.join('') + '</ul>', function () { up_them(good_files); }, null);
+            return modal.confirm(msg.join('') + '</ul>', function () {
+                up_them(good_files);
+                toast.inf(15, L.u_unpt);
+            }, null);
 
         up_them(good_files);
     }
@@ -1060,7 +1045,7 @@ function up2k_init(subtle) {
             pvis.addfile([
                 uc.fsearch ? esc(entry.name) : linksplit(
                     entry.purl + uricom_enc(entry.name)).join(' '),
-                '📐 hash',
+                '📐 ' + L.u_hashing,
                 ''
             ], fobj.size, draw_each);
 
@@ -1082,7 +1067,7 @@ function up2k_init(subtle) {
     function more_one_file() {
         fdom_ctr++;
         var elm = mknod('div');
-        elm.innerHTML = '<input id="file{0}" type="file" name="file{0}[]" multiple="multiple" />'.format(fdom_ctr);
+        elm.innerHTML = '<input id="file{0}" type="file" name="file{0}[]" multiple="multiple" tabindex="-1" />'.format(fdom_ctr);
         ebi('u2form').appendChild(elm);
         ebi('file' + fdom_ctr).onchange = gotfile;
     }
@@ -1108,11 +1093,11 @@ function up2k_init(subtle) {
         }
 
         if (!nhash)
-            ebi('u2etah').innerHTML = 'Done ({0}, {1} files)'.format(humansize(st.bytes.hashed), pvis.ctr["ok"] + pvis.ctr["ng"]);
+            ebi('u2etah').innerHTML = L.u_etadone.format(humansize(st.bytes.hashed), pvis.ctr["ok"] + pvis.ctr["ng"]);
 
         if (!nsend && !nhash)
             ebi('u2etau').innerHTML = ebi('u2etat').innerHTML = (
-                'Done ({0}, {1} files)'.format(humansize(st.bytes.uploaded), pvis.ctr["ok"] + pvis.ctr["ng"]));
+                L.u_etadone.format(humansize(st.bytes.uploaded), pvis.ctr["ok"] + pvis.ctr["ng"]));
 
         if (!st.busy.hash.length && !hashing_permitted())
             nhash = 0;
@@ -1133,7 +1118,7 @@ function up2k_init(subtle) {
         }
         if ((nhash || nsend) && !uc.fsearch) {
             if (!st.bytes.finished) {
-                ebi('u2etat').innerHTML = '(preparing to upload)';
+                ebi('u2etat').innerHTML = L.u_etaprep;
             }
             else {
                 st.time.busy += td;
@@ -1146,7 +1131,7 @@ function up2k_init(subtle) {
                 eta = Math.floor(rem / bps);
 
             if (t[a][1] < 1024 || t[a][3] < 0.1) {
-                ebi(t[a][0]).innerHTML = '(preparing to upload)';
+                ebi(t[a][0]).innerHTML = L.u_etaprep;
                 continue;
             }
 
@@ -1173,7 +1158,7 @@ function up2k_init(subtle) {
         var t = st.todo.handshake[0],
             cd = t.cooldown;
 
-        if (cd && cd - Date.now() > 0)
+        if (cd && cd > Date.now())
             return false;
 
         // keepalive or verify
@@ -1264,24 +1249,21 @@ function up2k_init(subtle) {
                     donut.on(is_busy);
 
                     if (!is_busy) {
-                        var k = uc.fsearch ? 'searches' : 'uploads',
-                            ks = uc.fsearch ? 'Search' : 'Upload',
-                            tok = uc.fsearch ? 'successful (found on server)' : 'completed successfully',
-                            tng = uc.fsearch ? 'failed (NOT found on server)' : 'failed, sorry',
+                        var sr = uc.fsearch,
                             ok = pvis.ctr["ok"],
                             ng = pvis.ctr["ng"],
                             t = uc.ask_up ? 0 : 10;
 
                         if (ok && ng)
-                            toast.warn(t, 'Finished, but some {0} failed:\n{1} {2},\n{3} {4}'.format(k, ok, tok, ng, tng));
+                            toast.warn(t, (sr ? L.ur_sm : L.ur_um).format(ok, ng));
                         else if (ok > 1)
-                            toast.ok(t, 'All {1} {0} {2}'.format(k, ok, tok));
+                            toast.ok(t, (sr ? L.ur_aso : L.ur_auo).format(ok));
                         else if (ok)
-                            toast.ok(t, '{0} {1}'.format(ks, tok));
+                            toast.ok(t, sr ? L.ur_1so : L.ur_1uo);
                         else if (ng > 1)
-                            toast.err(t, 'All {1} {0} {2}'.format(k, ng, tng));
+                            toast.err(t, (sr ? L.ur_asn : L.ur_aun).format(ng));
                         else if (ng)
-                            toast.err(t, '{0} {1}'.format(ks, tng));
+                            toast.err(t, sr ? L.ur_1sn : L.ur_1un);
 
                         timer.rm(etafun);
                         timer.rm(donut.do);
@@ -1350,7 +1332,8 @@ function up2k_init(subtle) {
                 }
 
                 if (st.todo.upload.length &&
-                    st.busy.upload.length < parallel_uploads) {
+                    st.busy.upload.length < parallel_uploads &&
+                    can_upload_next()) {
                     exec_upload();
                     mou_ikkai = true;
                 }
@@ -1370,6 +1353,14 @@ function up2k_init(subtle) {
         return taskerd;
     })();
 
+    function chill(t) {
+        var now = Date.now();
+        if ((t.coolmul || 0) < 2 || now - t.cooldown < t.coolmul * 700)
+            t.coolmul = Math.min((t.coolmul || 0.5) * 2, 32);
+
+        t.cooldown = Math.max(t.cooldown || 1, Date.now() + t.coolmul * 1000);
+    }
+
     /////
     ////
     ///   hashing
@@ -1468,7 +1459,6 @@ function up2k_init(subtle) {
                     min_filebuf = 1;
                     var td = Date.now() - t0;
                     if (td > 50) {
-                        ebi('u2foot').innerHTML += "<p>excessive filereader latency (" + td + " ms), increasing readahead</p>";
                         min_filebuf = 32 * 1024 * 1024;
                     }
                 }
@@ -1527,7 +1517,7 @@ function up2k_init(subtle) {
 
                 t.t_hashed = Date.now();
 
-                pvis.seth(t.n, 2, 'hashing done');
+                pvis.seth(t.n, 2, L.u_hashdone);
                 pvis.seth(t.n, 1, '📦 wait');
                 apop(st.busy.hash, t);
                 st.todo.handshake.push(t);
@@ -1655,8 +1645,8 @@ function up2k_init(subtle) {
 
                     if (!response || !response.hits || !response.hits.length) {
                         smsg = '404';
-                        msg = ('not found on server <a href="#" onclick="fsearch_explain(' +
-                            (has(perms, 'write') ? '0' : '1') + ')" class="fsearch_explain">(explain)</a>');
+                        msg = (L.u_s404 + ' <a href="#" onclick="fsearch_explain(' +
+                            (has(perms, 'write') ? '0' : '1') + ')" class="fsearch_explain">(' + L.u_expl + ')</a>');
                     }
                     else {
                         smsg = 'found';
@@ -1684,6 +1674,8 @@ function up2k_init(subtle) {
                     return;
                 }
 
+                t.sprs = response.sprs;
+
                 var rsp_purl = url_enc(response.purl);
                 if (rsp_purl !== t.purl || response.name !== t.name) {
                     // server renamed us (file exists / path restrictions)
@@ -1731,7 +1723,7 @@ function up2k_init(subtle) {
                             'npart': t.postlist[a]
                         });
 
-                    msg = 'uploading';
+                    msg = L.u_upping;
                     done = false;
 
                     if (sort)
@@ -1756,8 +1748,12 @@ function up2k_init(subtle) {
 
                     pvis.move(t.n, 'ok');
                 }
-                else t.t_uploaded = undefined;
+                else {
+                    if (t.t_uploaded)
+                        chill(t);
 
+                    t.t_uploaded = undefined;
+                }
                 tasker();
             }
             else {
@@ -1804,11 +1800,8 @@ function up2k_init(subtle) {
                     tasker();
                     return;
                 }
-                toast.err(0, "server broke; hs-err {0} on file [{1}]:\n".format(
-                    xhr.status, t.name) + (
-                        (xhr.response && xhr.response.err) ||
-                        (xhr.responseText && xhr.responseText) ||
-                        "no further information"));
+                err = t.t_uploading ? L.u_ehsfin : t.srch ? L.u_ehssrch : L.u_ehsinit;
+                xhrchk(xhr, err + ";\n\nfile: " + t.name + "\n\nerror ", "404, target folder not found");
             }
         }
         xhr.onload = function (e) {
@@ -1834,6 +1827,20 @@ function up2k_init(subtle) {
     ///   upload
     //
 
+    function can_upload_next() {
+        var upt = st.todo.upload[0],
+            upf = st.files[upt.nfile];
+
+        if (upf.sprs)
+            return true;
+
+        for (var a = 0, aa = st.busy.upload.length; a < aa; a++)
+            if (st.busy.upload[a].nfile == upt.nfile)
+                return false;
+
+        return true;
+    }
+
     function exec_upload() {
         var upt = st.todo.upload.shift();
         st.busy.upload.push(upt);
@@ -1867,9 +1874,9 @@ function up2k_init(subtle) {
                 console.log("ignoring dupe-segment error", t);
             }
             else {
-                toast.err(0, "server broke; cu-err {0} on file [{1}]:\n".format(
-                    xhr.status, t.name) + (txt || "no further information"));
-                return;
+                xhrchk(xhr, L.u_cuerr2.format(npart, Math.ceil(t.size / chunksize), t.name), "404, target folder not found (???)");
+
+                chill(t);
             }
             orz2(xhr);
         }
@@ -1896,7 +1903,7 @@ function up2k_init(subtle) {
                     return;
 
                 if (!toast.visible)
-                    toast.warn(9.98, "failed to upload a chunk;\nprobably harmless, continuing\n\n" + t.name);
+                    toast.warn(9.98, L.u_cuerr.format(npart, Math.ceil(t.size / chunksize), t.name));
 
                 console.log('chunkpit onerror,', ++tries, t);
                 orz2(xhr);
@@ -1920,6 +1927,7 @@ function up2k_init(subtle) {
     //
 
     function onresize(e) {
+        // 10x faster than matchMedia('(min-width
         var bar = ebi('ops'),
             wpx = window.innerWidth,
             fpx = parseInt(getComputedStyle(bar)['font-size']),
@@ -1929,22 +1937,19 @@ function up2k_init(subtle) {
             parent = ebi(wide && write ? 'u2btn_cw' : 'u2btn_ct'),
             btn = ebi('u2btn');
 
-        //console.log([wpx, fpx, wem]);
         if (btn.parentNode !== parent) {
             parent.appendChild(btn);
-            ebi('u2conf').setAttribute('class', wide);
-            ebi('u2cards').setAttribute('class', wide);
-            ebi('u2etaw').setAttribute('class', wide);
+            ebi('u2conf').className = ebi('u2cards').className = ebi('u2etaw').className = wide;
         }
 
         wide = write && wem > 78 ? 'ww' : wide;
         parent = ebi(wide == 'ww' && write ? 'u2c3w' : 'u2c3t');
         var its = [ebi('u2etaw'), ebi('u2cards')];
         if (its[0].parentNode !== parent) {
-            ebi('u2conf').setAttribute('class', wide);
+            ebi('u2conf').className = wide;
             for (var a = 0; a < 2; a++) {
                 parent.appendChild(its[a]);
-                its[a].setAttribute('class', wide);
+                its[a].className = wide;
             }
         }
     }
@@ -2011,17 +2016,18 @@ function up2k_init(subtle) {
     }
 
     function draw_turbo() {
-        var msgu = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;client may not detect and resume incomplete uploads; see turbo-button tooltip</span></p>',
-            msgs = '<p class="warn">WARNING: turbo enabled, <span>&nbsp;search results can be incorrect; see turbo-button tooltip</span></p>',
-            msg = uc.fsearch ? msgs : msgu,
-            omsg = uc.fsearch ? msgu : msgs,
+        var msg = uc.fsearch ? L.u_ts : L.u_tu,
+            omsg = uc.fsearch ? L.u_tu : L.u_ts,
             html = ebi('u2foot').innerHTML,
             ohtml = html;
 
-        if (uc.turbo && html.indexOf(msg) === -1)
+        if (turbolvl || !uc.turbo)
+            msg = null;
+
+        if (msg && html.indexOf(msg) === -1)
             html = html.replace(omsg, '') + msg;
-        else if (!uc.turbo)
-            html = html.replace(msgu, '').replace(msgs, '');
+        else if (!msg)
+            html = html.replace(L.u_tu, '').replace(L.u_ts, '');
 
         if (html !== ohtml)
             ebi('u2foot').innerHTML = html;
@@ -2057,13 +2063,15 @@ function up2k_init(subtle) {
 
         try {
             var ico = uc.fsearch ? '🔎' : '🚀',
-                desc = uc.fsearch ? 'Search' : 'Upload';
+                desc = uc.fsearch ? L.ul_btns : L.ul_btnu;
 
             clmod(ebi('op_up2k'), 'srch', uc.fsearch);
-            ebi('u2bm').innerHTML = ico + ' <sup>' + desc + '</sup>';
+            ebi('u2bm').innerHTML = ico + '&nbsp; <sup>' + desc + '</sup>';
         }
         catch (ex) { }
 
+        ebi('u2tab').className = (uc.fsearch ? 'srch ' : 'up ') + pvis.act;
+
         draw_turbo();
         onresize();
     }
