@@ -7,6 +7,7 @@ if (!window['console'])
 
 
 var is_touch = 'ontouchstart' in window,
+    is_https = (window.location + '').indexOf('https:') === 0,
     IPHONE = is_touch && /iPhone|iPad|iPod/i.test(navigator.userAgent),
     WINDOWS = navigator.platform ? navigator.platform == 'Win32' : /Windows/.test(navigator.userAgent);
 
@@ -14,7 +15,8 @@ var is_touch = 'ontouchstart' in window,
 var ebi = document.getElementById.bind(document),
     QS = document.querySelector.bind(document),
     QSA = document.querySelectorAll.bind(document),
-    mknod = document.createElement.bind(document);
+    mknod = document.createElement.bind(document),
+    XHR = XMLHttpRequest;
 
 
 function qsr(sel) {
@@ -89,6 +91,9 @@ function vis_exh(msg, url, lineNo, columnNo, error) {
     if ((msg + '').indexOf('l2d.js') !== -1)
         return;  // `t` undefined in tapEvent -> hitTestSimpleCustom
 
+    if (!/\.js($|\?)/.exec('' + url))
+        return;  // chrome debugger
+
     var ekey = url + '\n' + lineNo + '\n' + msg;
     if (ignexd[ekey] || crashed)
         return;
@@ -249,6 +254,14 @@ if (!Element.prototype.closest)
         } while (el !== null && el.nodeType === 1);
     };
 
+if (!String.prototype.format)
+    String.prototype.format = function () {
+        var args = arguments;
+        return this.replace(/{(\d+)}/g, function (match, number) {
+            return typeof args[number] != 'undefined' ?
+                args[number] : match;
+        });
+    };
 
 // https://stackoverflow.com/a/950146
 function import_js(url, cb) {
@@ -312,7 +325,7 @@ function clmod(el, cls, add) {
 
     var n2 = n1.replace(re, ' ') + (add ? ' ' + cls : '');
 
-    if (!n1 == !n2)
+    if (n1 == n2)
         return false;
 
     el.className = n2;
@@ -327,11 +340,21 @@ function clgot(el, cls) {
     if (el.classList)
         return el.classList.contains(cls);
 
-    var lst = (el.getAttribute('class') + '').split(/ /g);
+    var lst = (el.className + '').split(/ /g);
     return has(lst, cls);
 }
 
 
+var ANIM = true;
+if (window.matchMedia) {
+    var mq = window.matchMedia('(prefers-reduced-motion: reduce)');
+    mq.onchange = function () {
+        ANIM = !mq.matches;
+    };
+    ANIM = !mq.matches;
+}
+
+
 function showsort(tab) {
     var v, vn, v1, v2, th = tab.tHead,
         sopts = jread('fsort', [["href", 1, ""]]);
@@ -448,7 +471,7 @@ function linksplit(rp, id) {
         q = '?' + q[1];
     }
 
-    if (rp && rp.charAt(0) == '/')
+    if (rp && rp[0] == '/')
         rp = rp.slice(1);
 
     while (rp) {
@@ -792,6 +815,14 @@ function sethash(hv) {
     }
 }
 
+function dl_file(url) {
+    console.log('DL [%s]', url);
+    var o = mknod('a');
+    o.setAttribute('href', url);
+    o.setAttribute('download', '');
+    o.click();
+}
+
 
 var timer = (function () {
     var r = {};
@@ -872,7 +903,7 @@ var tt = (function () {
     };
 
     r.getmsg = function (el) {
-        if (QS('body.bbox-open'))
+        if (IPHONE && QS('body.bbox-open'))
             return;
 
         var cfg = sread('tooltips');
@@ -1172,6 +1203,9 @@ var modal = (function () {
             return ok();
         }
 
+        if ((k == 'ArrowLeft' || k == 'ArrowRight') && eng && (ae == eok || ae == eng))
+            return (ae == eok ? eng : eok).focus() || ev(e);
+
         if (k == 'Escape')
             return ng();
     }
@@ -1373,3 +1407,18 @@ var favico = (function () {
     r.to = setTimeout(r.init, 100);
     return r;
 })();
+
+
+function xhrchk(xhr, prefix, e404) {
+    if (xhr.status < 400 && xhr.status >= 200)
+        return true;
+
+    if (xhr.status == 403)
+        return toast.err(0, prefix + (window.L && L.xhr403 || "403: access denied\n\ntry pressing F5, maybe you got logged out"));
+
+    if (xhr.status == 404)
+        return toast.err(0, prefix + e404);
+
+    return toast.err(0, prefix + xhr.status + ": " + (
+        (xhr.response && xhr.response.err) || xhr.responseText));
+}
