@@ -38,6 +38,9 @@ class Cfg(Namespace):
             no_mv=False,
             no_del=False,
             no_zip=False,
+            no_thumb=False,
+            no_athumb=False,
+            no_vthumb=False,
             no_voldump=True,
             no_scandir=False,
             no_sendfile=True,
@@ -52,9 +55,17 @@ class Cfg(Namespace):
             mth="",
             textfiles="",
             doctitle="",
+            html_head="",
+            lang="eng",
+            theme=0,
+            themes=0,
+            turbo=0,
+            logout=573,
             hist=None,
             no_idx=None,
             no_hash=None,
+            force_js=False,
+            no_robots=False,
             js_browser=None,
             css_browser=None,
             **{k: False for k in "e2d e2ds e2dsa e2t e2ts e2tsr no_acode".split()}
